@@ -1,6 +1,7 @@
-__version__ = '1.1.1'
+__version__ = '1.5.1'
 
 import ast
+import importlib
 import os
 import re
 from configparser import (
@@ -17,15 +18,24 @@ from typing import List
 
 from django.apps.registry import apps as registry_apps
 from django.conf import settings
-from django.template import Template
-from django.template.base import TokenType
-from django.template.loader_tags import IncludeNode
+from django.template import (
+    Origin,
+    Template,
+)
+from django.template.base import (
+    tag_re,
+    TokenType,
+)
+from django.template.loader_tags import (
+    IncludeNode,
+)
 from django.templatetags.i18n import (
     BlockTranslateNode,
     TranslateNode,
 )
 from django.utils.functional import Promise
 from gitignorefile import Cache
+
 from okrand._vendored.polib import (
     POEntry,
     pofile,
@@ -57,8 +67,21 @@ def get_conf(name, default=None):
     return config.get(name, default)
 
 
+def should_process_model(model, prefixes):
+    if not prefixes:
+        return True
+
+    for p in prefixes:
+        if (model.__module__ + '.' + model.__name__).startswith(p):
+            return True
+    return False
+
+
 def translations_for_all_models():
+    prefixes = get_conf_list('django_model_prefixes')
     for model in registry_apps.get_models():
+        if not should_process_model(model=model, prefixes=prefixes):
+            continue
         yield from translations_for_model(model)
 
 
@@ -70,10 +93,16 @@ def translations_for_model(model):
 def _translations_for_model_only(model):
     verbose_name = getattr(model._meta.verbose_name, '_okrand_original_string', None)
     verbose_name_plural = getattr(model._meta.verbose_name_plural, '_okrand_original_string', None)
-    if verbose_name is not None or verbose_name_plural is not None:
+    if verbose_name is not None:
         yield String(
             msgid=verbose_name,
-            msgid_plural=verbose_name_plural,
+            translation_function='gettext',
+            domain='django',
+        )
+
+    if verbose_name_plural is not None:
+        yield String(
+            msgid=verbose_name_plural,
             translation_function='gettext',
             domain='django',
         )
@@ -147,7 +176,7 @@ def normalize_func(func):
     return func
 
 
-def parse_python(content):
+def parse_python(content, full_path=None):
     t = ast.parse(content)
 
     def w(node):
@@ -287,7 +316,7 @@ ml_languages_find_string_regex = r'''(?x)
     '''
 
 
-def parse_js(content):
+def parse_js(content, full_path=None):
     # I would like to have a proper JS parser here instead of a regex, but I couldn't find one that I could use in a reasonable way
     # An idea is to use @babel/parse to parse the file and dump the strings. But this would make node and babel a dependency. This is how far I got before I decided to stop:
     # const fs = require('fs');
@@ -330,7 +359,7 @@ def parse_js(content):
             assert False, f'unknown gettext flavor {func}'
 
 
-def parse_elm(content):
+def parse_elm(content, full_path):
     for m in re.finditer(ml_languages_find_string_regex, content):
         s = m.groupdict()['string']
         func = normalize_func(m.groupdict()['func'])
@@ -384,10 +413,21 @@ def extract_string_from_blocktrans_tokens(tokens):
     return ''.join(result)
 
 
-def parse_django_template(content):
-    t = Template(content)
+def parse_django_template(content, full_path=None):
+    t = Template(content, origin=Origin(name=str(full_path), template_name=str(full_path)))
     t.child_nodelists = ("nodelist",)
 
+    translation_underscore_function = r'''_\(['"](.*)['"]\)'''
+
+    # Find `_()` calls in the template
+    for template_thing in re.finditer(tag_re, content):
+        for m in re.finditer(translation_underscore_function, template_thing.group(0)):
+            yield String(
+                msgid=m.group(1),
+                translation_function='template _()',
+                domain='django',
+            )
+
     def w(node):
         if isinstance(node, TranslateNode):
             yield String(
@@ -408,7 +448,7 @@ def parse_django_template(content):
             )
 
         for child_nodelist in node.child_nodelists:
-            nodelist = getattr(node, child_nodelist)
+            nodelist = getattr(node, child_nodelist, [])
             for n in nodelist:
                 yield from w(n)
 
@@ -423,6 +463,7 @@ def ignore_filename(full_path, *, ignore_list):
     return False
 
 
+# TODO: pluggable stuff here
 parse_function_by_extension = {
     '.py': parse_python,
     '.html': parse_django_template,
@@ -441,6 +482,11 @@ def find_source_strings(ignore_list):
     if get_conf('django_model_upgrade', '0') in ('1', 'true'):
         yield from translations_for_all_models()
 
+    for plugin in get_conf_list('find_source_strings_plugins'):
+        module_name, _, function_name = plugin.rpartition('.')
+        module = importlib.import_module(module_name)
+        yield from getattr(module, function_name)(ignore_list=ignore_list)
+
     for root, dirs, files in walk_respecting_gitignore(settings.BASE_DIR):
         for f in files:
             extension = Path(f).suffix
@@ -455,7 +501,7 @@ def find_source_strings(ignore_list):
             with open(full_path) as file:
                 content = file.read()
 
-            yield from parse_function_by_extension[extension](content)
+            yield from parse_function_by_extension[extension](content, full_path=full_path)
 
 
 POEntry.__repr__ = lambda self: f'<POEntry: {self.msgid}{" (obsolete)" if self.obsolete else ""}>'
@@ -484,6 +530,7 @@ def update_po_files(*, old_msgid_by_new_msgid=None, sort=None, languages=None) -
 
     strings = list(find_source_strings(ignore_list=ignore_list))
 
+    # noinspection PyTypeChecker
     result_fields = fields(UpdateResult)
 
     # dicts as a poor man's ordered set
@@ -527,10 +574,9 @@ def update_language(*, language_code, strings, sort='none', old_msgid_by_new_msg
         if sort == 'alphabetical':
             po_file.sort(key=lambda x: x.msgid)
 
-        try:
+        if po_file:
+            Path(po_file.fpath).parent.mkdir(parents=True, exist_ok=True)
             po_file.save()
-        except FileNotFoundError as e:
-            print(e)
 
         yield result
 
@@ -621,7 +667,7 @@ def _update_language(*, po_file, strings, old_msgid_by_new_msgid=None, domain) -
         for po_entry in newly_obsolete_po_entries:
             po_entry.obsolete = True
 
-    if not newly_obsolete_po_entries:
+    if not newly_obsolete_po_entries or get_conf('renames', '1') in ('0', 'false'):
         for s in new_strings:
             data = dict(
                 msgid=s.msgid,
@@ -662,9 +708,11 @@ def _update_language(*, po_file, strings, old_msgid_by_new_msgid=None, domain) -
         newly_obsolete_strings = [x.msgid for x in newly_obsolete_po_entries]
 
     newly_obsolete_strings_set = set(newly_obsolete_strings)
+    previously_obsolete_strings = [x.msgid for x in obsolete_po_entries if x.msgid not in newly_obsolete_strings_set]
+
     return UpdateResult(
         new_strings=[x.msgid for x in new_strings],
         newly_obsolete_strings=newly_obsolete_strings,
-        previously_obsolete_strings=[x.msgid for x in obsolete_po_entries if x.msgid not in newly_obsolete_strings_set],
+        previously_obsolete_strings=previously_obsolete_strings,
         domain=domain,
     )
